
'''
Project      : 
FilePath     : \OPENSOURCE\main.py
Descripttion : 
Author       : GDDG08
Date         : 2022-11-08 02:07:44
LastEditors  : GDDG08
LastEditTime : 2024-04-03 16:10:23
'''
import sys
import argparse
from .gui import GUI
from .yanhekt import YanHeKT


def configArgs():
    parser = argparse.ArgumentParser(description='GDDG08/YanHeKT_Downloader')
    parser.add_argument('courseID', type=int, help='Course ID of YanHeKT')

    group_lession = parser.add_argument_group('Lesson Selection', 'IF NONE, PRINT LESSON LIST AND EXIT.')
    group_lession_exc = group_lession.add_mutually_exclusive_group()
    group_lession_exc.add_argument('-A', '--all', action='store_true', help='Download all lessons')
    group_lession_exc.add_argument('-L', '--list', type=int, nargs='+', metavar='i', help='Select of lesson index (e.g., --list 1 2 4)')
    group_lession_exc.add_argument('-R', '--range', type=int, nargs=2, action='append', metavar='i',
                                   help='Select range of lessons (e.g., --range 3 5 for [3,5))')

    group_video = parser.add_argument_group('Video Type')
    group_video_exc = group_video.add_mutually_exclusive_group()
    group_video_exc.add_argument('-D', '--dual', action='store_true', help='Download both VGA(PC) and Video (default)')
    group_video_exc.add_argument('-G', '--vga', action='store_true', help='Download VGA(PC) only')
    group_video_exc.add_argument('-V', '--video', action='store_true', help='Download Video only')

    group_config = parser.add_argument_group('Configurations')
    group_config.add_argument('-S', '--skip', action='store_true', help='Skip existing files')
    group_config.add_argument('--dir', type=str, default='./', help='Output directory (e.g., --dir ./output)')
    group_config.add_argument('--max-workers', type=int, default=32, metavar='num', help='Max workers for downloading (default: 32)')

    return parser


def parseArgs(parser):
    # force to show help message if no arguments provided
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    else:
        args = parser.parse_args()
        # merge list and range arguments
        if not (args.vga or args.video):
            args.dual = True

        return args


def printArgs(args):
    print("="*10+" YanHeKT_Downloader "+"="*10)
    print(f"Course ID: {args.courseID}")
    print(f"Lesson Selection: {'ALL' if args.all else args.list or args.range}")
    print(f"Video Type: {'VGA & Video' if args.dual else 'VGA' if args.vga else 'Video'}")
    print(f"Skip existing files: {args.skip}")
    print(f"Output Directory: {args.dir}")
    print("="*40+"\n")


def progressPrint(current, total):
    sys.stdout.write('\r[%-25s](%d/%d)' % ("*" * (100 * current // total // 4),
                                           current, total))
    sys.stdout.flush()


def runCli():
    parser = configArgs()
    args = parseArgs(parser)
    printArgs(args)

    yanhekt = YanHeKT(args.courseID, args.all, args.list, args.range, args.dual, args.vga, args.video, args.skip, args.dir, args.max_workers)
    yanhekt.download(callback_prog=progressPrint)


def runGui():
    gui = GUI()
    gui.event_loop()


def main():
    # judge whether to run in GUI mode
    if len(sys.argv) > 1 and sys.argv[1] == 'gui':
        runGui()
    else:
        runCli()


if __name__ == "__main__":
    main()



'''
Project      :
FilePath     : \OPENSOURCE\m3u8dl.py
Descripttion :
Author       : GDDG08
Date         : 2022-11-07 12:09:26
LastEditors  : GDDG08
LastEditTime : 2024-04-03 15:39:46
'''
import os
import re
import sys
import queue
import base64
import platform
import requests
import urllib3
import time
from concurrent.futures import ThreadPoolExecutor
from hashlib import md5


class ThreadPoolExecutorWithQueueSizeLimit(ThreadPoolExecutor):
    """
    实现多线程有界队列
    队列数为线程数的2倍
    """

    def __init__(self, max_workers=None, *args, **kwargs):
        super().__init__(max_workers, *args, **kwargs)
        self._work_queue = queue.Queue(max_workers * 2)


def make_sum():
    ts_num = 0
    while True:
        yield ts_num
        ts_num += 1


class M3u8Download:
    """
    :param url: 完整的m3u8文件链接 如"https://www.bilibili.com/example/index.m3u8"
    :param name: 保存m3u8的文件名 如"index"
    :param max_workers: 多线程最大线程数
    :param num_retries: 重试次数
    :param base64_key: base64编码的字符串
    """

    def __init__(self, url, workDir, name, max_workers=64, num_retries=16, base64_key=None, callback_progress=None):

        self._url = url
        self._token = None
        self._name = name
        self._max_workers = max_workers
        self._num_retries = num_retries
        self._workDir = os.path.join(workDir, self._name)
        self._file_path = os.path.join(os.getcwd(), "temp", self._name)
        self._front_url = None
        self._ts_url_list = []
        self._success_sum = 0
        self._ts_sum = 0
        self._key = base64.b64decode(base64_key.encode()) if base64_key else None
        self._headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.82 Safari/537.36 Edg/93.0.961.52',
                         'Origin': 'https://www.yanhekt.cn',
                         'referer': 'https://www.yanhekt.cn',
                         }
        self._callback_progress = callback_progress

        urllib3.disable_warnings()

        self.sign_prefix = "1tJrMwNq3h0yLgx86Rued2J1tFc"
        self._url = self.encryptURL(self._url)
        # self.refreshSignature()

        self.get_m3u8_info(self._url, self._num_retries)
        # print('Downloading: %s' % self._name, 'Save path: %s' % self._file_path, sep='\n')
        with ThreadPoolExecutorWithQueueSizeLimit(self._max_workers) as pool:
            for k, ts_url in enumerate(self._ts_url_list):
                pool.submit(self.download_ts, ts_url, os.path.join(self._file_path, str(k)), self._num_retries)
        if self._success_sum == self._ts_sum:
            self.output_mp4()
            self.delete_file()
            print("SUCCESS")

    def getSignature(self):
        timestamp = str(int(time.time()))
        sign_org = "_".join([self.sign_prefix, 'v1', timestamp])
        signature = md5(sign_org.encode()).hexdigest()
        # print(timestamp, signature)
        return timestamp, signature

    def refreshSignature(self):
        self.token = self.getToken()
        self.timestamp, self.signature = self.getSignature()

    def encryptURL(self, url):
        sign_org = "_".join([self.sign_prefix, '100'])
        signature = md5(sign_org.encode()).hexdigest()

        index = url.rfind('/')
        if index > -1:
            l = url[:index]
            r = url[index:]
            return l + '/' + signature + r
        return url

    def getToken(self):
        if self._token == None:
            headers = {
                "Xdomain-Client": "web_user",
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.82 Safari/537.36 Edg/93.0.961.52',
                'Origin': 'https://www.yanhekt.cn',
            }

            req = requests.get("https://cbiz.yanhekt.cn/v1/auth/video/token?id=0",
                               headers=headers)
            data = req.json()['data']
            self._token = data['token']
        return self._token

    def get_m3u8_info(self, m3u8_url, num_retries):
        """
        获取m3u8信息
        """
        self.refreshSignature()
        url = m3u8_url+"?Xvideo_Token="+self.token+"&Xclient_Timestamp=" + self.timestamp+"&Xclient_Signature=" + \
            self.signature+"&Xclient_Version=v1&Platform=yhkt_user"
        try:
            with requests.get(url, timeout=(3, 30), verify=False, headers=self._headers) as res:
                self._front_url = res.url.split(res.request.path_url)[0]
                if "EXT-X-STREAM-INF" in res.text:  # 判定为顶级M3U8文件
                    for line in res.text.split('\n'):
                        if "#" in line:
                            continue
                        elif line.startswith('http'):
                            self._url = line
                        elif line.startswith('/'):
                            self._url = self._front_url + line
                        else:
                            self._url = self._url.rsplit("/", 1)[0] + '/' + line
                    self.get_m3u8_info(self._url, self._num_retries)
                else:
                    m3u8_text_str = res.text
                    self.get_ts_url(m3u8_text_str)
        except Exception as e:
            print(e)
            if num_retries > 0:
                self.get_m3u8_info(m3u8_url, num_retries - 1)

    def get_ts_url(self, m3u8_text_str):
        """
        获取每一个ts文件的链接
        """
        if not os.path.exists(self._file_path):
            os.makedirs(self._file_path)
        new_m3u8_str = ''
        ts = make_sum()
        for line in m3u8_text_str.split('\n'):
            if "#" in line:
                if "EXT-X-KEY" in line and "URI=" in line:
                    if os.path.exists(os.path.join(self._file_path, 'key')):
                        continue
                    key = self.download_key(line, 5)
                    if key:
                        new_m3u8_str += f'{key}\n'
                        continue
                new_m3u8_str += f'{line}\n'
                if "EXT-X-ENDLIST" in line:
                    break
            else:
                if line.startswith('http'):
                    self._ts_url_list.append(line)
                elif line.startswith('/'):
                    self._ts_url_list.append(self._front_url + line)
                else:
                    self._ts_url_list.append(self._url.rsplit("/", 1)[0] + '/' + line)
                new_m3u8_str += (os.path.join(self._file_path, str(next(ts))) + '\n')
        self._ts_sum = next(ts)
        with open(self._file_path + '.m3u8', "wb") as f:
            if platform.system() == 'Windows':
                # f.write(new_m3u8_str.encode('gbk'))
                f.write(new_m3u8_str.encode('utf-8'))
            else:
                f.write(new_m3u8_str.encode('utf-8'))

    def download_ts(self, ts_url, name, num_retries):
        """
        下载 .ts 文件
        """

        self.refreshSignature()
        ts_url = ts_url.split('\n')[0]+"?Xvideo_Token="+self.token+"&Xclient_Timestamp=" + \
            self.timestamp+"&Xclient_Signature="+self.signature+"&Xclient_Version=v1&Platform=yhkt_user"
        try:
            if not os.path.exists(name):
                with requests.get(ts_url, stream=True, timeout=(5, 60), verify=False, headers=self._headers) as res:
                    if res.status_code == 200:
                        with open(name, "wb") as ts:
                            for chunk in res.iter_content(chunk_size=1024):
                                if chunk:
                                    ts.write(chunk)
                        self._success_sum += 1
                        # sys.stdout.write('\r[%-25s](%d/%d)' % ("*" * (100 * self._success_sum // self._ts_sum // 4),
                        #                                        self._success_sum, self._ts_sum))
                        # sys.stdout.flush()
                        if self._callback_progress:
                            self._callback_progress(self._success_sum, self._ts_sum)
                    else:
                        self.refreshSignature()
                        self.download_ts(ts_url, name, num_retries - 1)
            else:
                self._success_sum += 1
        except Exception:
            if os.path.exists(name):
                print(name)
                os.remove(name)
            if num_retries > 0:
                self.download_ts(ts_url, name, num_retries - 1)

    def download_key(self, key_line, num_retries):
        """
        下载key文件
        """
        mid_part = re.search(r"URI=[\'|\"].*?[\'|\"]", key_line).group()
        may_key_url = mid_part[5:-1]
        if self._key:
            with open(os.path.join(self._file_path, 'key'), 'wb') as f:
                f.write(self._key)
            return f'{key_line.split(mid_part)[0]}URI="./{self._name}/key"'
        if may_key_url.startswith('http'):
            true_key_url = may_key_url
        elif may_key_url.startswith('/'):
            true_key_url = self._front_url + may_key_url
        else:
            true_key_url = self._url.rsplit("/", 1)[0] + '/' + may_key_url
        try:
            with requests.get(true_key_url, timeout=(5, 30), verify=False, headers=self._headers) as res:
                with open(os.path.join(self._file_path, 'key'), 'wb') as f:
                    f.write(res.content)
            return f'{key_line.split(mid_part)[0]}URI="./{self._name}/key"{key_line.split(mid_part)[-1]}'
        except Exception as e:
            print(e)
            if os.path.exists(os.path.join(self._file_path, 'key')):
                os.remove(os.path.join(self._file_path, 'key'))
            print("加密视频,无法加载key,揭秘失败")
            if num_retries > 0:
                self.download_key(key_line, num_retries - 1)

    def output_mp4(self):
        """
        合并.ts文件，输出mp4格式视频，需要ffmpeg
        """
        #
        cmd = f'''ffmpeg -y -loglevel quiet -allowed_extensions ALL -i "{self._file_path}.m3u8" -acodec \
        copy -vcodec copy -f mp4 "{self._workDir}.mp4"'''
        # print(cmd)
        os.system(cmd)

    def delete_file(self):
        file = os.listdir(self._file_path)
        for item in file:
            os.remove(os.path.join(self._file_path, item))
        os.removedirs(self._file_path)
        os.remove(self._file_path + '.m3u8')


if __name__ == "__main__":
    # url_list = input("输入url，若同时输入多个url时要用空格分开：").split()
    # name_list = input("输入name，若同时输入多个name要用空格分开：").split()
    # # 如果M3U8_URL的数量 ≠ SAVE_NAME的数量
    # # 下载一部电视剧时，只需要输入一个name就可以了
    # sta = len(url_list) == len(name_list)
    # for i, u in enumerate(url_list):
    #     M3u8Download(u,
    #                  name_list[i] if sta else f"{name_list[0]}{i + 1:02}",
    #                  max_workers=64,
    #                  num_retries=10,
    #                  # base64_key='5N12sDHDVcx1Hqnagn4NJg=='
    #                  )
    pass
    # M3u8Download("https://*.m3u8", "./", "test.mp4")




'''
Project      :
FilePath     : \OPENSOURCE\yanhekt.py
Descripttion :
Author       : GDDG08
Date         : 2022-11-08 02:07:44
LastEditors  : GDDG08
LastEditTime : 2024-04-03 16:03:17
'''

import requests
from .m3u8dl import M3u8Download
import os
headers = {
    'Origin': 'https://www.yanhekt.cn',
    "xdomain-client": "web_user",
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 Edg/107.0.1418.26'
}


class YanHeKT():
    def __init__(self, _courseID, _all=False, _list=None, _range=None, _dual=True, _vga=False, _video=False, _skip=False, _dir='./',  _max_workers=32) -> None:
        self.courseID = _courseID
        self.lessonList = None
        self.courseInfo = None

        self.updateArgs(_all, _list, _range, _dual, _vga, _video, _skip, _dir, _max_workers)

    def updateArgs(self, _all=False, _list=None, _range=None, _dual=True, _vga=False, _video=False, _skip=False, _dir='./',  _max_workers=32):
        self.all = _all
        self.list = _list
        self.range = _range
        self.dual = _dual
        self.vga = _vga
        self.video = _video
        self.skip = _skip
        self.dir = _dir
        self.max_workers = _max_workers

    def getCourseInfo(self):
        print("----Getting Course Information----")

        rqt_course = requests.get(f'https://cbiz.yanhekt.cn/v1/course?id={self.courseID}&with_professor_badges=true', headers=headers)
        courseInfo = rqt_course.json()['data']

        print(courseInfo['name_zh'])

        print("-----Getting Lesson List-----")
        rqt_list = requests.get(f'https://cbiz.yanhekt.cn/v2/course/session/list?course_id={self.courseID}', headers=headers)
        lessonList = rqt_list.json()['data']

        for i, lesson in enumerate(lessonList):
            print(f"[{i}] ", lesson['title'])

        self.courseInfo = courseInfo
        self.lessonList = lessonList
        return courseInfo, lessonList

    def download(self, callback=None, callback_prog=None):
        if not self.lessonList or not self.courseInfo:
            self.getCourseInfo()

        print("------Start Downloading------")

        selectList = []
        if self.all:
            selectList = list(range(len(self.lessonList)))
        elif self.list:
            selectList = self.list
        elif self.range:
            selectList += list(range(self.range[0][0], self.range[0][1]))
        else:
            print("[Error] No lesson selected in args.")
            print("Please use -A/--all, -L/--list, or -R/--range to select lessons.")
            print("Example:")
            print(f"\tpython main.py {self.courseID} --all")
            print(f"\tpython main.py {self.courseID} --list 0 2 4")
            print(f"\tpython main.py {self.courseID} --range 3 5")

            if callback:
                callback(False)
            return

        courseFullName = '-'.join([str(self.courseID), self.courseInfo['name_zh'], self.courseInfo['professors'][0]['name']])
        dirName = os.path.join(self.dir, courseFullName)

        if not os.path.exists(dirName):
            os.makedirs(dirName)

        for i in selectList:
            video = self.lessonList[i]
            fileName = video['title'].replace("/", "-")  # 防止文件名中的/导致路径错误

            print(f"Downloading {fileName} --->")

            videos = video['videos'][0]
            # 下载投影录屏
            if self.vga or self.dual:
                if 'vga' in videos:  # 检查是否存在vga链接
                    if self.skip and os.path.exists(f"{dirName}/{fileName}-VGA.mp4"):
                        print(f"VGA seems already done. Skipping...")
                    else:
                        print("VGA -->")
                        M3u8Download(videos['vga'], dirName, fileName + '-VGA', max_workers=self.max_workers, callback_progress=callback_prog)
                else:
                    print(f"No VGA found.")

            # 下载视频
            if self.video or self.dual:
                if 'main' in videos:  # 检查是否存在main链接
                    if self.skip and os.path.exists(f"{dirName}/{fileName}-Video.mp4"):
                        print(f"Video seems already done. Skipping...")
                    else:
                        print("Video -->")
                        M3u8Download(videos['main'], dirName, fileName + '-Video', max_workers=self.max_workers, callback_progress=callback_prog)
                else:
                    print(f"No Video found.")

        if callback:
            callback(True)
        return


if __name__ == '__main__':
    # main()
    # yanhekt = YanHeKT(12345, _all=True, _dir='./')
    # yanhekt.download(callback_prog=progressPrint)
    pass



'''
Project      : 
FilePath     : \PypiPkg\src\yanhekt\__init__.py
Descripttion : 
Author       : GDDG08
Date         : 2024-04-02 17:54:53
LastEditors  : GDDG08
LastEditTime : 2024-04-02 17:54:59
'''
from .yanhekt import YanHeKT



'''
Project      : 
FilePath     : \OPENSOURCE\gui.py
Descripttion : 
Author       : GDDG08
Date         : 2024-04-03 12:53:30
LastEditors  : GDDG08
LastEditTime : 2024-04-03 16:45:43
'''
import re
import threading
from .yanhekt import YanHeKT
import PySimpleGUI as sg


class GUI:
    def __init__(self):
        self.window = None
        self.init_ui()
        self.yanhekt = None
        self.td = None

    def init_ui(self):
        # sg.theme('DarkAmber')  # Set a theme for the GUI
        sg.theme('DarkTeal7')

        frame_course_info = [

            [
                sg.Column([
                            [sg.Text('Web Link:', expand_x=True), sg.Input(key='WEB_LINK',
                                                                           default_text="https://www.yanhekt.cn/course/11111", enable_events=True, size=(35, 2), expand_x=True)],
                            [sg.Text('Course ID:', expand_x=True), sg.Input(key='COURSE_ID', enable_events=True, size=(35, 2), expand_x=True)]
                ]),
                sg.Column([
                    # [sg.Button('Get Course ID', expand_x=True, disabled=True)],
                    [sg.Button('Get Course Info', expand_x=True, disabled=True)]
                ])
            ],
            [sg.HorizontalSeparator()],
            [
                sg.Column([
                    [sg.Text('Course Name:'), sg.Text('', key='COURSE_NAME'),  sg.Text('Professor:'), sg.Text('', key='PROFESSOR')],
                    [sg.Listbox(values=[], size=(50, 8), expand_x=True, key='LESSON_LIST',
                                enable_events=True, select_mode=sg.LISTBOX_SELECT_MODE_EXTENDED)],
                    [sg.Text('use Ctrl or Shift or Drag to select multiple')],
                    [sg.Text('Ctrl + A to select all')],
                ], pad=(25, 0), expand_x=True, element_justification='center')],
        ]

        # frame_lesson_selection = [
        #     [sg.Radio('All', group_id="Selection", key='ALL', default=True)],
        #     [sg.Radio('List', group_id="Selection", key='LIST')],
        #     [sg.Checkbox('Lesson 1', key='LESSON_1'), sg.Checkbox('Lesson 2', key='LESSON_2'), sg.Checkbox('Lesson 3', key='LESSON_3')],
        #     [sg.Radio('Range', group_id="Selection", key='RANGE')],
        #     [sg.Text('from'), sg.Spin([i for i in range(1, 10)], initial_value=0, key='RANGE_START'),
        #      sg.Text('to'), sg.Spin([i for i in range(1, 10)], initial_value=1, key='RANGE_END')],

        # ]

        frame_settings = [
            [sg.Text('Video Type:'), sg.Radio('Dual', group_id="RADIO_VIDEO_TYPE", key='DUAL', default=True), sg.Radio(
                'VGA', group_id="RADIO_VIDEO_TYPE", key='VGA'), sg.Radio('Video', group_id="RADIO_VIDEO_TYPE", key='VIDEO')],
            [sg.Text('Output Directory:'), sg.Input("./Downloads", size=(30, 2), key="OUTPUT_DIR", expand_x=True),
             sg.FolderBrowse(key='OUTPUT_DIR_BROWSE', initial_folder="./")],
            [sg.Checkbox('Skip Existing', default=True, key='SKIP_EXISTING')],
        ]

        # Layout definition
        layout = [
            [sg.Frame('Course Information', frame_course_info)],
            # [sg.Frame('Lesson Selection', frame_lesson_selection)],
            [sg.Frame('Settings', frame_settings, expand_x=True)],

            [sg.Column([
                [sg.Button('Download Lessons', disabled=True)],
                [sg.Output(size=(60, 10), expand_x=True, key='OUTPUT')],
                [sg.ProgressBar(100, orientation='h', size=(40, 10), expand_x=True, key='PROGRESS_BAR')]
            ], expand_x=True,element_justification='center')]
            
        ]

        # Create the Window
        self.window = sg.Window('YanHeKT Downloader', layout)

    def event_loop(self):
        while True:
            event, values = self.window.read()
            if event == sg.WIN_CLOSED:  # If user closes window
                if self.td:
                    self.td.stop()
                break
            try:
                # if event == 'Get Course ID':
                #     courseID = values['WEB_LINK'].split('/')[-1]
                #     if (courseID.isdigit()):
                #         self.window['COURSE_ID'].update(courseID)
                #         self.window['Get Course Info'].update(disabled=False)
                if event == 'Get Course Info':
                    if not values['COURSE_ID'].isdigit():
                        sg.popup_error("Invalid Course ID")
                        continue
                    else:
                        self.window['Download Lessons'].update(disabled=True)

                        self.yanhekt = YanHeKT(values['COURSE_ID'])
                        courseInfo, lessonList = self.yanhekt.getCourseInfo()
                        self.window['COURSE_NAME'].update(courseInfo['name_zh'])
                        self.window['PROFESSOR'].update(courseInfo['professors'][0]['name'])

                        lessonTitles = [lesson['title'] for lesson in lessonList]
                        self.window['LESSON_LIST'].update(values=[tittle for tittle in lessonTitles])
                elif event == 'Download Lessons':
                    self.window['Download Lessons'].update(disabled=True)
                    self.window['PROGRESS_BAR'].update(0)

                    self.window['OUTPUT'].update("Downloading...\n\n")
                    # print(values)
                    lessonTitles = [lesson['title'] for lesson in self.yanhekt.lessonList]

                    selected_lessons = [i for i, tittle in enumerate(lessonTitles) if tittle in values['LESSON_LIST']]
                    self.yanhekt.updateArgs(_all=False, _list=selected_lessons, _range=False,
                                            _dual=values['DUAL'], _vga=values['VGA'], _video=values['VIDEO'], _skip=values['SKIP_EXISTING'], _dir=values['OUTPUT_DIR'])
                    self.do_download()
                elif event == 'WEB_LINK':
                    # print(values['WEB_LINK'])
                    if values['WEB_LINK']:
                        # self.window['Get Course ID'].update(disabled=False)

                        rex = re.search(r'(?<=course/)(\d+)', values['WEB_LINK'])

                        if (rex):
                            self.window['COURSE_ID'].update(rex.group(0))
                            self.window['Get Course Info'].update(disabled=False)
                        else:
                            self.window['COURSE_ID'].update("")
                            self.window['Get Course Info'].update(disabled=True)
                    # else:
                    #     self.window['Get Course ID'].update(disabled=True)
                elif event == 'COURSE_ID':
                    if values['COURSE_ID']:
                        self.window['Get Course Info'].update(disabled=False)
                    else:
                        self.window['Get Course Info'].update(disabled=True)
                elif event == 'LESSON_LIST':
                    if (len(values['LESSON_LIST']) > 0):
                        self.window['Download Lessons'].update(disabled=False)
                    else:
                        self.window['Download Lessons'].update(disabled=True)

            except Exception as e:
                sg.popup_error(f'An error occurred: {str(e)}')

        self.window.close()

    def do_download(self):
        # new thread
        self.td = threading.Thread(target=self.yanhekt.download, args=(self.callback_download, self.callback_progress))
        self.td.start()

    def callback_download(self, isFinished):
        self.window['Download Lessons'].update(disabled=False)
        # self.window['OUTPUT'].update(f"Download success? {isFinished}\n\n")

    def callback_progress(self, current, total):
        self.window['PROGRESS_BAR'].update(current/total*100)
